package org.netkernel.layer0.meta.impl;

import java.util.*;

import org.netkernel.container.IMessages;
import org.netkernel.layer0.bnf.BNFGrammarFactory;
import org.netkernel.layer0.bnf.GrammarParseException;
import org.netkernel.layer0.bnf.IGrammarGroup;
import org.netkernel.layer0.meta.GrammarException;
import org.netkernel.layer0.meta.IEndpointArgumentMeta;
import org.netkernel.layer0.meta.IEndpointMeta;
import org.netkernel.layer0.meta.IIdentifierGrammar;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestResponseFields;
import org.netkernel.request.impl.RequestResponseFieldsImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.IMetaRepresentation;


public class EndpointMetaBuilder
{
	private String mName;
	private String mDescription;
	private IIdentifier mIcon;
	private IIdentifier mDoc;
	protected IRequestResponseFields mFields=RequestResponseFieldsImpl.EMPTY;
	
	protected int mVerbs=0;
	protected boolean mVerbsSet=false;
	protected IIdentifierGrammar mGrammar;
	protected IGrammarGroup mGrammarRoot;
	protected Map mArguments;
	protected List mResponseClasses;
	protected List mExceptions;
	
	public static IIdentifierGrammar DEFAULT_GRAMMAR;
	static
	{	try
		{	IGrammarGroup g = BNFGrammarFactory.createGroup(null);
			BNFGrammarFactory.createRegex(g, BNFGrammarFactory.BUILTIN_NOTHING);
			DEFAULT_GRAMMAR = BNFGrammarFactory.createGrammar(g,null);
		} catch (GrammarParseException e)
		{
		}
	}
	// BNFGrammarFactory.createRegex(mGrammarRoot, BNFGrammarFactory.BUILTIN_ANYTHING);

	//protected IMessages mMessages;
	
	public EndpointMetaBuilder()
	{	//mMessages=aMessages;
	}
	
	public void setName(String aName)
	{	mName=aName;
	}
	
	public boolean isNameSet()
	{	return mName!=null;
	}
	
	public void setDescription(String aDescription)
	{	mDescription=aDescription;
	}
	
	public void setIcon(IIdentifier aIcon)
	{	mIcon=aIcon;
	}
	
	public void setHumanReadableDocumentation(IIdentifier aDoc)
	{	mDoc=aDoc;
	}
	
	public void addField(String aName, Object aValue)
	{
		if (mFields==RequestResponseFieldsImpl.EMPTY)
		{	mFields = new RequestResponseFieldsImpl(4);
		}
		((RequestResponseFieldsImpl)mFields).put(aName, aValue);
	}
	
	public void setVerbs(int aVerbs)
	{	mVerbs=aVerbs;
		mVerbsSet=true;
	}
	
	public boolean areVerbsSet()
	{	return mVerbsSet;
	}
	
	public void setGrammar(IIdentifierGrammar aGrammar)
	{	mGrammar=aGrammar;
	}
	
	public void setGrammarRoot(IGrammarGroup aGrammarRoot)
	{	mGrammarRoot=aGrammarRoot;
	}
	
	public void addArgument(IEndpointArgumentMeta aArgument)
	{	if (mArguments==null)
		{	mArguments=new HashMap(16);
		}
		mArguments.put(aArgument.getName(),aArgument);
	}
	
	public void addResponseClass(Class aResponseClass)
	{	if (mResponseClasses==null)
		{	mResponseClasses=new ArrayList(4);
		}
		mResponseClasses.add(aResponseClass);
	}
	
	public void addException(String aExceptionId, String aDescription)
	{	if (mExceptions==null)
		{	mExceptions=new ArrayList(4);
		}
		IMetaRepresentation exceptionMeta = new MetaRepresentationImpl(aExceptionId,aDescription,null,null);
		mExceptions.add(exceptionMeta);
	}
	
	public IIdentifierGrammar getGrammar(IMessages aMessages) throws GrammarParseException
	{	IIdentifierGrammar g;
		if(mGrammar==null)
		{	if (mGrammarRoot==null)
			{	g=DEFAULT_GRAMMAR;
			}
			else
			{	g=BNFGrammarFactory.createGrammar(mGrammarRoot,aMessages);
			}
		}
		else
		{	g=mGrammar;
		}
		return g;
	}
	
	public IEndpointMeta getMeta(IMessages aMessages) throws GrammarParseException
	{	mGrammar=getGrammar(aMessages);
		
		IEndpointArgumentMeta[] arguments;
		
		if (mArguments!=null)
		{	arguments = new IEndpointArgumentMeta[mArguments.size()];
			int j=0;
			for (Iterator i=mArguments.values().iterator(); i.hasNext(); )
			{	arguments[j++]=(IEndpointArgumentMeta)i.next();
			}
		}
		else
		{	arguments=EndpointMetaImpl.NO_ARGS;
		}
		
		Class[] responseClasses;
		if (mResponseClasses!=null)
		{	responseClasses=new Class[mResponseClasses.size()];
			for (int i=0; i<responseClasses.length; i++)
			{	responseClasses[i]=(Class)mResponseClasses.get(i);
			}
		}
		else
		{	responseClasses=EndpointMetaImpl.NO_RESPONSES;
		}
		
		IMetaRepresentation[] exceptions;
		if (mExceptions!=null)
		{	exceptions=new IMetaRepresentation[mExceptions.size()];
			for (int i=0; i<exceptions.length; i++)
			{	exceptions[i]=(IMetaRepresentation)mExceptions.get(i);
			}
		}
		else
		{	exceptions=EndpointMetaImpl.NO_EXCEPTIONS;
		}
		EndpointMetaImpl em=new EndpointMetaImpl(mName,mDescription,mIcon,mDoc,mVerbs,mGrammar,arguments,responseClasses,exceptions);
		em.setAdditionalFields(mFields);
		return em;
	}
}
