/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.meta.IEndpointMeta;
import org.netkernel.request.IExpiry;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponse;
import org.netkernel.urii.INetKernelException;
import org.netkernel.urii.IResolution;
import org.netkernel.urii.ISpace;

/**
 * <b>Advanced</b> Methods which provide Low-level access to the NetKernel kernel APIs. Generally not necessary when implementing an accessor
 * but can be valuable for advanced development.
 * @author  tab
 */
public interface INKFKernelContext
{   
	/** <b>Advanced</b> Return the request which initiated this invocation of the accessor
	 * or null if there is no parent request (for example in a transport)
	 */
	IRequest getThisKernelRequest();
	
	/** <b>Advanced</b> Issue a resource request to the kernel using a URRequest.  All higher-level
	 *  NKF methods will use this to issue kernel requests.  Requests are Synchronous.
	 * @param aRequest
	 * @exception NetKernelException
	 * @return a representation which is the response from the request
	 */
	IResponse issueKernelRequest(IRequest aRequest) throws INetKernelException;
	
	/**  <b>Advanced</b> Issue a request that to all intents and purposes looks like a transport
	 * request. Known usage is to create a request that the visualizer can capture in the unit-test framework
	 * @param aRequest
	 * @return
	 * @throws NKFException
	 */
	INKFResponseReadOnly issueRootRequestForResponse(INKFRequest aRequest) throws NKFException;
	
	/** <b>Advanced</b> Issue a request that to all intents and purposes looks like a transport
	 * request. Known usage is to create a request that the visualizer can capture in the unit-test framework
	 *@param aRequest
	 *@return 
	 */
	INKFAsyncRequestHandle issueAsyncRootRequest(INKFRequest aRequest) throws NKFException;
	
	/** <b>Advanced</b> Return a reference to the Kernel Container. From this the Scheduler and other
	 * kernel components may be accessed.
	 */
	IKernel getKernel();
	
	/** <b>Advanced</b> Return the owning space of this endpoint
	 */
	ISpace getOwningSpace();
	
	/** <b>Advanced</b> If an endpoint locally caches a response from a request between
	 * invocations, the effects of that request in terms of scope, cost and expiry are not
	 * automatically propagated on the response of subsequent uses. This method should
	 * be called when subsequent uses of a response are made. 
	 * @param aResponse A response that has been returned from a request in a previous
	 * invocation of the endpoint and has been locally cached.
	 * @exception NKFException thrown if response is from a request that depended upon
	 * the parents request scope.
	 */
	void declareDependency(INKFResponseReadOnly aResponse) throws NKFException;
	
	/** <b>Advanced</b> Force a response to be expired regardless of how response is set later
	 *  - maybe useful for getParts and such where response isn't every set but dependencies accumulated 
	 */
	void declareExpiryDependency();
	
	/** <b>Advanced</b> - adjust the scope of the eventual response after know scope manipulation
	 */
	public void declareScopeDependency(int aScope);
	
	/** <b>Advanced</b> - adjust the scope and expiry of the eventual response
	 */
	void declareDependency(IExpiry aExpiry, int aScope);
	
	/** Perform a raw resolution request for a kernel IResolution without actually issuing
	 * the request
	 * @param aRequest
	 * @return
	 * @throws NKFException
	 */
	IResolution resolveRequest(INKFRequest aRequest) throws NKFException;
	
	/** Create an INKFResponseReadOnly from a raw kernel response
	 * @param aResponse
	 * @return
	 */
	INKFResponseReadOnly wrapKernelResponse(IResponse aResponse);
	
	/**Constructs a request to an endpoint indicated by the supplied endpoint metadata
	 * @param aMeta
	 * @return
	 * @throws NKFException
	 */
	INKFRequest createRequestToEndpoint(IEndpointMeta aMeta) throws NKFException;
	
	/** Return the request scope that requests will be created and issued with by default
	 * @return
	 */
	IRequestScopeLevel getRequestScope();
	
	/** Sets the request scope that requests will be created and issued with by default
	 * @param aScope
	 */
	void setRequestScope(IRequestScopeLevel aScope);
		
}