/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf;

import java.util.Iterator;

import org.netkernel.request.IRequestResponseFields;


/** The full response returned from a subrequest including expiry and metadata
 * @author tab
 */
public interface INKFResponseReadOnly<R>
{
	public static final String MIME_UNKNOWN="content/unknown";
	
	/** Return the request that was issued to give this response
	 */
	INKFRequest getRequest();
	
	/** Return the representation in the response
	 */
	R getRepresentation();
	
	/** Return the expiry determinant of this response
	 */
	boolean isExpired();
	
	/** Return an iterator over the names of the headers
	 */
	Iterator<String> getHeaderNames();
	
	/** Return true if a header with the given name is present on the response
	 * @param aKey
	 */
	boolean hasHeader(String aName);
	
	/** Return header value
	 * @param aName the name of the header to return
	 * @return a representation of the header value
	 * @throws NKFException if header is not found 
	 */
	Object getHeader(String aName) throws NKFException;
	
	/** Return header value with a specified representation class. A transreption
	 *  will occur if necessary
	 * @param aName the key of the header to return
	 * @param aRepresentationClass the class which the header must implement
	 * @throws NKFException if header is not found of transreption fails
	 */
	<T> T getHeaderAs(String aKey, Class<T> aRepresentationClass) throws NKFException;
	
	/** Return the raw header structure
	 * @return
	 */
	IRequestResponseFields getHeaders();
	
	/* Get mimetype, defaults to MIME_UNKNOWN if not set
	 */
	String getMimeType();
}