/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf.impl;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.urii.ParsedIdentifierImpl;
import org.netkernel.request.*;
import org.netkernel.urii.IIdentifier;

import java.util.*;
/**
 * Wrapper around the request that initiated the invocation of the accessor
 * @author  tab
 */
public class NKFRequestReadOnlyImpl implements INKFRequestReadOnly
{
	private IRequest mRequest;
	protected IIdentifier mIdentifier;
	private NKFContextImpl mContext;
	
	/** Creates a new instance of NKFRequestReadOnlyImpl */
	public NKFRequestReadOnlyImpl(IRequest aRequest)
	{	mRequest = aRequest;
		mIdentifier=aRequest.getIdentifier();
	}
	
	void setContext(NKFContextImpl aContext)
	{	mContext=aContext;
	}
	
	protected ParsedIdentifierImpl getParsed()
	{	if (!(mIdentifier instanceof ParsedIdentifierImpl))
		{	mIdentifier=ParsedIdentifierImpl.parseActiveSyntax(mIdentifier.toString());
		}
		return (ParsedIdentifierImpl)mIdentifier;
	}
	
	public IRequest getKernelRequest()
	{	return mRequest;
	}
	
	public Class getRepresentationClass()
	{	return mRequest.getRepresentationClass();
	}
	
	public int getVerb()
	{	return mRequest.getVerb();
	}
	
	public String getIdentifier()
	{	return mIdentifier.toString();
	}
	
	public boolean argumentExists(String aName)
	{	return getParsed().getArgumentValue(aName)!=null;
	}
	
	public String getArgumentValue(String aName)
	{	return getParsed().getArgumentValue(aName);
	}
	
	public Iterator getArguments()
	{	throw new IllegalArgumentException("not implemented- INKFRequestReadOnly.getArguments() deprecated");
	}

	public int getArgumentCount()
	{	return getParsed().getArgumentCount();
	}

	public String getArgumentName(int aIndex)
	{	return getParsed().getArgumentKey(aIndex);
	}

	public String getArgumentValue(int aIndex)
	{	return getParsed().getArgumentValue(aIndex);
	}

	public Object getPrimary()
	{	IResponse response = (IResponse)mRequest.getValue(IRequest.TYPE_SOURCED_RESPONSE);
		return (response!=null) ? response.getRepresentation() : null;
	}

	public INKFResponseReadOnly getPrimaryAsResponse()
	{	IResponse response = (IResponse)mRequest.getValue(IRequest.TYPE_SOURCED_RESPONSE);
		return (response!=null) ? new NKFResponseReadOnlyImpl(response,null,mContext) : null;
	}
	
	public String toString()
	{	return mRequest.toString();
	}

	public String getResolvedElementId()
	{	IIdentifier id=(IIdentifier)mRequest.getValue(IRequest.TYPE_RESOLVED_ENDPOINT_IDENTIFIER);
		String result=null;
		if (id!=null)
		{	result=id.toString();
		}
		return result;
	}
	
	public Object getHeaderValue(String aKey)
	{	return mRequest.getHeaders().getValue(aKey);
	}
	
	public List<Object> getHeaderValues(String aKey)
	{	return mRequest.getHeaders().getValues(aKey);
	}
	
	public Set<String> getHeaderKeys()
	{	IRequestResponseFields headers = mRequest.getHeaders();
		if (headers.size()>0)
		{	Set<String> result = new HashSet<String>(headers.size());
			for (int i=0; i<headers.size(); i++)
			{	result.add(headers.getKey(i));
			}
			return result;
		}
		else
		{	return Collections.EMPTY_SET;
		}
	}

	public INKFRequest getIssuableClone() throws NKFException
	{
		NKFRequestImpl req= (NKFRequestImpl)mContext.createRequest(this.getIdentifier());
		req.setVerb(this.getVerb());
		req.setRepresentationClass(this.getRepresentationClass());
		INKFResponseReadOnly primaryResponse = this.getPrimaryAsResponse();
		if (primaryResponse!=null)
		{	req.addPrimaryArgumentFromResponse(primaryResponse);
		}
		IRequestResponseFields headers=mRequest.getHeaders();
		for (int i=0; i<headers.size(); i++)
		{	req.setHeader(headers.getKey(i), headers.getValue(i), headers.isSticky(i));
		}
		return req;
	}
	
}