/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf.impl;


import org.netkernel.container.IKernel;
import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.util.Utils;
import org.netkernel.request.*;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.request.impl.RequestFactory;
import org.netkernel.request.impl.ResponseImpl;
import org.netkernel.urii.*;
import org.netkernel.util.ExclusionSet;



/**
 * The main base-class for creating spaces using the NetKernel Foundation API
 * @author  tab
 */
public abstract class NKFSpaceImpl implements ISpace
{
	private IKernel mKernel;

	public void onAsyncRequest(IRequest aRequest)
    {	IResponse response=null;
		try
		{	if (mKernel==null)
			{	// can't use generated message here because don't have a handle to kernel!
				throw new NKFException("Space not commissioned",this.getClass().getName());
			}
		
			NKFRequestReadOnlyImpl nkfRequest=null;
			if (aRequest!=null)
			{	nkfRequest = new NKFRequestReadOnlyImpl(aRequest);
			}
			if (aRequest.getVerb()==INKFRequestReadOnly.VERB_RESOLVE)
			{	ExclusionSet excluded = (ExclusionSet)aRequest.getValue(IRequest.TYPE_RESOLVE_EXCLUDED_SPACES);
				if (!excluded.contains(this))
				{	NKFResolutionContextImpl context = new NKFResolutionContextImpl(mKernel,this,nkfRequest,this);
					onResolve(context);
					context.pop();
					excluded.add(this);
					response = context.getKernelResponse();
				}
				else
				{	response = new ResponseImpl(aRequest,null,MetaImpl.EXPIRED);
				}
			}		
			else
			{	NKFResolutionContextImpl context = new NKFResolutionContextImpl(mKernel,this,nkfRequest,this);
				unsupportedVerb(context);
			}		
		}
		catch (Exception e)
		{	// unhandled exceptions have dependent expiry
			response = new ResponseImpl(aRequest,e,MetaImpl.EXPIRED); // changed to expire because this might be because of condition that
			// can be retried
		}
		if (response!=null)
		{	aRequest.getRequestor().onResponse(response);
		}
    }
	
	public void onResolve(INKFResolutionContext aContext) throws Exception
	{	unsupportedVerb(aContext);
	}

	public ClassLoaderWithExports getClassLoader()
	{	return null;
	}
	
	public boolean isCommissioned()
	{	return mKernel!=null;
	}
	
	public void onLinkSpace(IKernel aKernel) throws Exception
	{	// nothing to do by default
	}

	public void onCommissionSpace(IKernel aKernel) throws Exception
	{	mKernel=aKernel;
	}

	public void postCommissionSpace() throws Exception
	{	// nothing to do by default
	}
	
	public void preDecommissionSpace() throws Exception
	{	// nothing to do by default
	}	
	
	public void onDecommissionSpace() throws Exception
	{	mKernel=null;
	}
	
	protected IKernel getKernel()
	{	return mKernel;
	}
	
	protected void unsupportedVerb(INKFRequestContext aContext) throws Exception
	{	String endpointClass = this.getClass().getName();
		String verb=RequestFactory.verbString(aContext.getThisRequest().getVerb());
		throw Utils.createFormattedException("EX_NKF_VERB_NOT_SUPPORTED", "MSG_NKF_VERB_NOT_SUPPORTED", aContext.getKernelContext().getKernel().getLogger(), verb,endpointClass);
	}
	
	public boolean resolveEquals(Object aOther)
	{	return equals(aOther);
	}	
	
}