package org.netkernel.layer0.representation.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.IHDSPredicate;
import org.netkernel.layer0.util.HDSXPath;

public class HDSNodeListImpl implements IHDSNodeList
{
	private final List<IHDSNode> mNodes;
	
	public static final IHDSNodeList EMPTY = new HDSNodeListImpl(Collections.EMPTY_LIST);
	
	public HDSNodeListImpl(List<IHDSNode> aNodes)
	{	mNodes=aNodes;
	}

	public IHDSNodeList filter(IHDSPredicate aPredicate)
	{	int len=mNodes.size();
		List<IHDSNode> nodes = new ArrayList<IHDSNode>(len);
		for (int i=0; i<len; i++)
		{	IHDSNode n=mNodes.get(i);
			if (aPredicate.matches(n))
			{	nodes.add(n);
			}
		}
		return new HDSNodeListImpl(nodes);
	}

	public IHDSNode get(int aIndex)
	{	return mNodes.get(aIndex);
	}

	public IHDSNode getFirstNode(String aXPath)
	{	IHDSNode result=null;
		int len=mNodes.size();
		for (int i=0; i<len && result==null; i++)
		{	result = HDSXPath.getSingleNode(mNodes.get(i), aXPath,false);
		}
		return result;
	}

	public IHDSNode getFirstNode()
	{	if (mNodes.size()>0)
		{	return mNodes.get(0);
		}
		else
		{	return null;
		}
	}

	public Object getFirstValue(String aXPath)
	{	IHDSNode result=null;
		int len=mNodes.size();
		for (int i=0; i<len && result==null; i++)
		{	result = HDSXPath.getSingleNode(mNodes.get(i), aXPath,false);
		}
		if (result!=null)
		{	return result.getValue();
		}
		else
		{	return null;
		}
	}

	public Object getFirstValue()
	{	if (mNodes.size()>0)
		{	return mNodes.get(0).getValue();
		}
		else
		{	return null;
		}
	}

	public IHDSNodeList getNodes(String aXPath)
	{	int len=mNodes.size();
		List<IHDSNode> nodes = new ArrayList<IHDSNode>(len);
		for (int i=0; i<len; i++)
		{	HDSXPath.eval(nodes, mNodes.get(i), aXPath);
		}
		return new HDSNodeListImpl(nodes);
	}

	public int size()
	{	return mNodes.size();
	}

	public Iterator<IHDSNode> iterator()
	{	return new Iterator<IHDSNode>()
		{
			private int mIndex=0;
			public boolean hasNext()
			{	return mIndex<mNodes.size();
			}

			public IHDSNode next()
			{	if (hasNext())
				{	return mNodes.get(mIndex++);
				}
				else
				{	throw new NoSuchElementException();
				}
			}

			public void remove()
			{	throw new UnsupportedOperationException();
			}
		};
	}

	public IHDSNodeList sort(java.util.Comparator<IHDSNode> aComparator)
	{	List<IHDSNode> nodes=new ArrayList<IHDSNode>(mNodes);
		Collections.sort(nodes,aComparator);
		return new HDSNodeListImpl(nodes);
	}


	public String toString()
	{	StringBuffer sb=new StringBuffer();
    
        Iterator iter = mNodes.iterator();
        while (iter.hasNext())
            {
            IHDSNode node = (IHDSNode)iter.next();
            sb.append(node.getName());
            sb.append(" : ");
            sb.append(node.getValue().toString());
            sb.append("\n");
            }
		return sb.toString();
	}


}
