package org.netkernel.layer0.urii;

import java.io.File;

import org.netkernel.layer0.nkf.INKFExpiryFunction;

/** An expiry function for a java.io.File that will expire if the file is modifed, created or deleted
 * @author Tony Butterfield
 *
 */
public class FileExpiry implements INKFExpiryFunction
{
	private final File mFile;
	private final long mExpectedTimestamp;
	private final long mPollPeriod;
	private long mNextCheck;
	
	/**
	 * @param aFile the file to check
	 * @param aPollPeriod how often to recheck
	 */
	public FileExpiry(File aFile, long aPollPeriod)
	{	mFile=aFile;
		mExpectedTimestamp=mFile.lastModified();
		mPollPeriod=aPollPeriod;
		mNextCheck=1;
	}
	
	public boolean isExpired(long aNow)
	{	boolean result;
		if (mNextCheck==0)
		{	result = true;
		}
		else if (mNextCheck>aNow )
		{	result = false;
		}
		else
		{	long lastModified = mFile.lastModified();
			// a zero value if the file has been deleted
			result = lastModified>mExpectedTimestamp || (lastModified==0 && mExpectedTimestamp!=0);
			
			if (result)
			{	mNextCheck=0;
			}
			else
			{	mNextCheck = aNow+mPollPeriod;
			}
		}
		return result;
	}
	
	public String toString()
	{	return mFile.getAbsolutePath();
	}
}
