/******************************************************************************
 * (c) Copyright 2002,2003, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.util;
import java.util.regex.*;

/**
 * Regular Expression based URL Rewriter Rule.
 * 
 *	eg  Maps  Regex --> New
 *
 * Capturing groups are substituted into New using the apache
 * mod_rewrite $1...$N convention.
 *
 * @author  pjr
 */
public class RegexRewriterRule
{
	private Matcher mMatcher;
	private Matcher mToMatcher;
	private String mTo;
	
	private static Pattern sGroupPattern=Pattern.compile("\\$(e?\\d+)");
	
	/** Creates a new instance of RegexRewriterRule.
	 *	Matches are created and cached to reduce wasted objects.
	 */		
	public RegexRewriterRule(String aPattern, String aTo)
	{	mMatcher=Pattern.compile(aPattern).matcher("");
		mTo=aTo;
		mToMatcher=sGroupPattern.matcher(aTo);
	}

	/**
	 * Map a URI using the preconfigured Matchers
	 */
	 public String map(String aURI)
	 {	String result = aURI;
	 	boolean matched;
	 	synchronized(mMatcher)
		{	mMatcher.reset(aURI);
			matched=mMatcher.matches();
			if(matched)
			{	String replace=mTo;
				StringBuffer sb=new StringBuffer(replace.length()*2);
				mToMatcher.reset(replace);
				while (mToMatcher.find())
				{	String toMatch = mToMatcher.group(1);
					char action = ' ';
					if (!Character.isDigit(toMatch.charAt(0)))
					{	action=toMatch.charAt(0);
						toMatch = toMatch.substring(1);
					}
					int j=Integer.parseInt(toMatch);
					String replacement = mMatcher.group(j);
					switch (action)
					{	case 'e':
							replacement = encode(replacement);
							break;
					}
					if (replacement.indexOf('$')>=0)
					{	//we must escape all $ signs with $$ so that appendReplacement works as expected
						int len=replacement.length();
						char[] rep2 = new char[len+len];
						int k=0;
						for (int i=0; i<len; i++)
						{	char c=replacement.charAt(i);
							if (c=='$')
							{	rep2[k++]='\\';
							}
							rep2[k++]=c;
						}
						replacement=new String(rep2,0,k);
					}
					mToMatcher.appendReplacement(sb, replacement);
				 }
				 mToMatcher.appendTail(sb);
				result = sb.toString();
			}
		}
		return result;
	 }
	
	/** simple test to see if it matches without doing anything
	 */
	public boolean matches(CharSequence aString)
	{	synchronized(mMatcher)
		{	mMatcher.reset(aString);
			return mMatcher.matches();
		}
	}
	
	private static String encode(String aInput)
	{	StringBuffer sb=new StringBuffer(aInput.length()*3/2);
		int length = aInput.length();
		for (int i=0; i<length; i++)
		{	char c = aInput.charAt(i);
			switch (c)
			{	case ' ': sb.append("%20"); break;
				case '%': sb.append("%25"); break;
				case ':': sb.append("%3A"); break;
				case '#': sb.append("%23"); break;
				case '+': sb.append("%2B"); break;
				case '@': sb.append("%40"); break;
				case '=': sb.append("%3D"); break;
				default: sb.append(c);
			}
		}
		return new String(sb);
	}	
	
	public String toString()
	{	return mTo;
	}
	
}
